/**
 * @file snow_effect.h
 */

#ifndef _SNOW_EFFECT
#define _SNOW_EFFECT

#include "vector.h"
#include "particle_system.h"
#include "material.h"
#include "vector2D.h"

#include "weather_effect.h"

class SpriteParticles;
class PlaneEmitter;
class PNode;

class SnowEffect : public WeatherEffect
{
  public:
    SnowEffect(const TiXmlElement* root = NULL);
    virtual ~SnowEffect();

    virtual void loadParams(const TiXmlElement* root);

    virtual bool init();

    virtual bool activate();
    virtual bool deactivate();

    virtual void draw() const;
    virtual void tick(float dt);

    void activateSnow();
    void deactivateSnow();

  private:
    void numParticles(int n) { this->particles = n; }
    void materialTexture(const std::string& texture) { this->texture = texture; }
    void lifeSpan(float lifeSpan, float randomLifeSpan) { this->life = lifeSpan; this->randomLife = randomLifeSpan; }
    void radius(float radius, float randomRadius) { this->snowRadius = radius; this->randomRadius = randomRadius; }
    void mass(float mass, float randomMass) { this->snowMass = mass; this->randomMass = randomMass; }
    void emissionRate(float emissionRate){ this->rate = emissionRate; }
    void emissionVelocity(float velocity, float randomVelocity){ this->velocity = velocity; this->randomVelocity = randomVelocity; }
    void spread(float angle, float randomAngle) { this->angle = angle; this->randomAngle = randomAngle; }
    void size(float sizeX, float sizeY) { this->snowSize = Vector2D(sizeX, sizeY); }
    void coord(float x, float y, float z) { this->snowCoord = Vector(x, y, z); }

    int particles;
    std::string texture;
    float life, randomLife;
    float snowRadius, randomRadius;
    float snowMass, randomMass;
    float rate;
    float velocity, randomVelocity;
    float angle, randomAngle;
    float alpha;
    Vector snowCoord;
    Vector2D snowSize;

    bool activated;

    static SpriteParticles* snowParticles;

    PlaneEmitter*          emitter;
};


#endif  /* _SNOW_EFFECT */
